# Copyright 2018-present Lenovo
# Confidential and Proprietary

import base64
import falcon
import os
import sys
import traceback

from adapter import get_infe_serv
from falcon.media.validators import jsonschema
from tempfile import NamedTemporaryFile


class DeployServiceRunner(object):
    def __init__(self, scenario, deploy_path, model_name):
        self.deploy_path = deploy_path
        self.model_path = os.path.join(deploy_path, 'model')
        self.InferenceServiceMgt = get_infe_serv(
            scenario, self.model_path, model_name
        )
        sys.stdout.flush()

    @jsonschema.validate(
        {
            "type": "object",
            "properties": {
                "image": {"type": "string"},
                "image_type": {
                    "type": "string",
                    "enum": ["BASE64"]
                },
            },
            "required": ["image", "image_type"]
        }
    )
    def on_post(self, req, resp):
        try:
            content = base64.b64decode(req.media.get('image'))
            with NamedTemporaryFile(mode='wb') as buf:
                buf.write(content)
                buf.flush()
                inf_result = self.InferenceServiceMgt.inference(buf.name)
        except ValueError:
            traceback.print_exc()
            raise falcon.HTTPBadRequest(
                description='The format of image is unsupported'
            )
        except TypeError:
            traceback.print_exc()
            raise falcon.HTTPBadRequest(
                description='It is not an image'
            )
        except Exception:
            traceback.print_exc()
            raise falcon.HTTP_INTERNAL_SERVER_ERROR(
                description='Internal error of inference server'
            )
        resp.media = inf_result
